@@ -95,6 +95,12 @@ class Meta(type):
                 def f(a, b):
                     self = a
                     # print(op, a, b)
+                    if self.df.is_category(self.expression) and self.df._encode_categoricals and not isinstance(b, Expression):
+                        labels = self.df.category_labels(self.expression)
+                        if b not in labels:
+                            raise ValueError(f'Value {b} not present in {labels}')
+                        b = labels.index(b)
+                        a = self.index_values()
                     try:
                         stringy = isinstance(b, str) or b.is_string()
                     except:
@@ -106,6 +112,8 @@ class Meta(type):
                             b = repr(b)
                         if op['code'] == '==':
                             expression = 'str_equals({0}, {1})'.format(a.expression, b)
+                        elif op['code'] == '!=':
+                            expression = 'str_notequals({0}, {1})'.format(a.expression, b)
                         elif op['code'] == '+':
                             expression = 'str_cat({0}, {1})'.format(a.expression, b)
                         else:
@@ -193,11 +201,27 @@ class Expression(with_metaclass(Meta)):
         assert not isinstance(ds, Expression)
         if isinstance(expression, Expression):
             expression = expression.expression
+        if expression is None and ast is None:
+            raise ValueError('Not both expression and the ast can be None')
         self._ast = ast
         self._expression = expression
         self.df._expressions.append(weakref.ref(self))
         self._ast_names = None
 
+    @property
+    def _label(self):
+        '''If a column is an invalid identified, the expression is df['long name']
+        This will return 'long name' in that case, otherwise simply the expression
+        '''
+        ast = self.ast
+        if isinstance(ast, expresso._ast.Subscript):
+            value = ast.slice.value
+            if isinstance(value, expresso.ast_Str):
+                return value.s
+            if isinstance(value, str):  # py39+
+                return value
+        return self.expression
+
     def copy(self, df=None):
         """Efficiently copies an expression.
 
@@ -864,6 +888,7 @@ def f({0}):
         """
         if use_hashmap:
             # easiest way to create a set is using the vaex dataframe
+            values = np.array(values, dtype=self.dtype.numpy)  # ensure that values are the same dtype as the expression (otherwise the set downcasts at the C++ level during execution)
             df_values = vaex.from_arrays(x=values)
             ordered_set = df_values._set(df_values.x)
             var = self.df.add_variable('var_isin_ordered_set', ordered_set, unique=True)
@@ -872,7 +897,7 @@ def f({0}):
             if self.is_string():
                 values = vaex.column._to_string_sequence(values)
             else:
-                values = np.array(values, dtype=self.dtype)
+                values = np.array(values, dtype=self.dtype.numpy)
             var = self.df.add_variable('isin_values', values, unique=True)
             return self.df['isin(%s, %s)' % (self, var)]
 
